<?php
/**
 * Conditional Display Add-On For Bricks Theme
 *
 * Plugin Name: Conditional Display Add-On For Bricks Theme
 * Plugin URI:  https://itchycode.com/create-my-custom-conditional-visibility-feature-in-bricks-theme-without-add-on/
 * Description: Temporary add-on for Bricks Theme to display content based on simple logic.
 * Version:     1.0.4
 * Author:      Jenn Lee
 * Author URI:  https://itchycode.com/
 * License:     GPLv2 or later
 * Text Domain: itchy-bricks
 * Domain Path: /languages
 * Requires at least: 6.0
 * Requires PHP: 7.4
 *
 */

if (!defined('WPINC')) {
    die;
}

if (!class_exists('Itchy_Bricks_Conditional_Display')) :

    define('ITCHY_BCD_VERSION', '1.0.4');
    define('ITCHY_BCD_PATH', plugin_dir_path(__FILE__));
    define('ITCHY_BCD_URL', plugin_dir_url(__FILE__));

    class Itchy_Bricks_Conditional_Display
    {

        public static function init_plugin()
        {
            self::load_textdomain();
            $pass = self::check_requirement();

            if ($pass) {
                add_action('init', array(__CLASS__, 'add_controls_to_all_elements'), 30);
                add_filter('bricks/element/render', array(__CLASS__, 'conditional_render'), 20, 2);
            }

        }

        public static function load_textdomain()
        {
            load_plugin_textdomain('itchy-bricks', false, basename(dirname(__FILE__)) . '/languages');
        }

        public static function check_requirement()
        {

            $theme = wp_get_theme();
            $active_theme_name = ($theme->parent()) ? $theme->parent()->get('Name') : $theme->get('Name');
            $active_theme_version = ($theme->parent()) ? $theme->parent()->get('Version') : $theme->get('Version');

            if ($active_theme_name !== 'Bricks') {
                add_action('admin_notices', array(__CLASS__, 'bricks_theme_required_notice'));
                return false;
            }

            if (version_compare($active_theme_version, '1.4.0.1', '<')) {
                add_action('admin_notices', array(__CLASS__, 'bricks_version_required_notice'));
                return false;
            }

            return true;
        }

        public static function bricks_version_required_notice()
        {

            $class = 'notice notice-error';
            $message = __('Itchy Bricks Conditional Display Add-On requires Bricks Theme higher than 1.4.0.1', 'itchy-bricks');
            printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), esc_html($message));

        }

        public static function bricks_theme_required_notice()
        {

            $class = 'notice notice-error';
            $message = __('Itchy Bricks Conditional Display Add-On requires Bricks Theme.', 'itchy-bricks');
            printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), esc_html($message));

        }

        public static function add_controls_to_all_elements()
        {

            $elements = \Bricks\Elements::$elements;
            $names = array_keys($elements);
            foreach ($names as $name) {
                add_filter("bricks/elements/{$name}/control_groups", array(__CLASS__, 'add_control_groups'), 10);
                add_filter("bricks/elements/{$name}/controls", array(__CLASS__, 'add_controls'), 10);
            }

        }

        public static function add_control_groups($control_groups)
        {

            $control_groups['itchy_advanced'] = [
                'tab'   => 'style',
                'title' => esc_html__('Itchy Advanced', 'itchy-bricks'),
            ];

            return $control_groups;

        }

        public static function add_controls($controls)
        {

            // This is a checkbox to activate or deactivate the condition visibility feature on the element
            $controls['itchy_condition_active'] = [
                'tab'   => 'style',
                'group' => 'itchy_advanced',
                'label' => esc_html__('Use Conditional Visibility', 'itchy-bricks'),
                'type'  => 'checkbox'
            ];

            // This is a dropdown to determine show or hide if the condition is met
            $controls['itchy_condition_met'] = [
                'tab'         => 'style',
                'group'       => 'itchy_advanced',
                'label'       => esc_html__('Conditional Type', 'itchy-bricks'),
                'type'        => 'select',
                'options'     => [
                    'show' => esc_html__('Show', 'itchy-bricks'),
                    'hide' => esc_html__('Hide', 'itchy-bricks'),
                ],
                'placeholder' => esc_html__('Choose a condition type', 'itchy-bricks'),
                'required'    => ['itchy_condition_active', '!=', ''],
            ];

            // This capture value A
            $controls['itchy_condition_value_a'] = [
                'tab'         => 'style',
                'group'       => 'itchy_advanced',
                'label'       => esc_html__('Conditional Value A', 'itchy-bricks'),
                'type'        => 'text',
                'placeholder' => '',
                'required'    => ['itchy_condition_active', '!=', ''],
                'description' => esc_html__('This will be compare with Value B', 'itchy-bricks'),
            ];

            // This decide the compare operator
            $controls['itchy_condition_compare'] = [
                'tab'      => 'style',
                'group'    => 'itchy_advanced',
                'label'    => esc_html__('Compare', 'itchy-bricks'),
                'type'     => 'select',
                'options'  => [
                    '='  => esc_html__('Equal', 'itchy-bricks'),
                    '!=' => esc_html__('Not equal', 'itchy-bricks'),
                    '>'  => esc_html__('Greater than', 'itchy-bricks'),
                    '>=' => esc_html__('Greater than or equal', 'itchy-bricks'),
                    '<'  => esc_html__('Lesser', 'itchy-bricks'),
                    '<=' => esc_html__('Lesser or equal', 'itchy-bricks'),
                ],
                'required' => ['itchy_condition_active', '!=', ''],
            ];

            // This capture value B
            $controls['itchy_condition_value_b'] = [
                'tab'         => 'style',
                'group'       => 'itchy_advanced',
                'label'       => esc_html__('Conditional Value B', 'itchy-bricks'),
                'type'        => 'text',
                'placeholder' => '',
                'required'    => ['itchy_condition_active', '!=', ''],
                'description' => esc_html__('This will be compare with Value A', 'itchy-bricks'),
            ];

            // This going to convert the value A and B to a desire data type before compare
            $controls['itchy_condition_data_type'] = [
                'tab'         => 'style',
                'group'       => 'itchy_advanced',
                'label'       => esc_html__('Compare Data Type', 'itchy-bricks'),
                'type'        => 'select',
                'options'     => [
                    'number'       => esc_html__('Number', 'itchy-bricks'),
                    'string'       => esc_html__('String', 'itchy-bricks'),
                    'boolean'      => esc_html__('Boolean', 'itchy-bricks'),
                    'stringlength' => esc_html__('String Length', 'itchy-bricks'),
                ],
                'description' => esc_html__('Convert Value A and Value B before compare', 'itchy-bricks'),
                'required'    => ['itchy_condition_active', '!=', ''],
            ];


            return $controls;
        }


        public static function conditional_render($show, $element)
        {

            $settings = $element->settings;

            // If nothing set, return original value
            if (
                !isset($settings['itchy_condition_active']) ||
                !isset($settings['itchy_condition_met']) ||
                !isset($settings['itchy_condition_value_a']) ||
                !isset($settings['itchy_condition_value_b']) ||
                !isset($settings['itchy_condition_compare']) ||
                !isset($settings['itchy_condition_data_type']) ||
                !$element->is_frontend
            ) return $show;

            $active = $settings['itchy_condition_active'];
            $met = $settings['itchy_condition_met'];
            $value_a = $settings['itchy_condition_value_a'];
            $value_b = $settings['itchy_condition_value_b'];
            $compare = $settings['itchy_condition_compare'];
            $data_type = $settings['itchy_condition_data_type'];

            // Ensure all required values are set
            if (
                !$active ||
                $met == '' ||
                $value_a == '' ||
                $value_b == '' ||
                $compare == '' ||
                $data_type == ''
            ) return $show;

            // Value A and Value B might be using dynamic data, so we need to get the actual value
            $value_a = $element->render_dynamic_data($value_a);
            $value_b = $element->render_dynamic_data($value_b);

            // Let's convert the value to the correct desired compare type
            switch ($data_type) {
                case 'number':
                    $value_a = floatval($value_a);
                    $value_b = floatval($value_b);
                    break;
                case 'string':
                    $value_a = strval($value_a);
                    $value_b = strval($value_b);
                    break;
                case 'boolean':
                    $value_a = filter_var($value_a, FILTER_VALIDATE_BOOLEAN);
                    $value_b = filter_var($value_b, FILTER_VALIDATE_BOOLEAN);
                    break;
                case 'stringlength':
                    $value_a = strlen(strval($value_a));
                    $value_b = strlen(strval($value_b));
                    break;
                default:
                    $value_a = $value_a;
                    $value_b = $value_b;
                    break;
            }

            // Let's compare the values!
            switch ($compare) {
                case '=':
                    $show = $value_a == $value_b;
                    break;
                case '!=':
                    $show = $value_a != $value_b;
                    break;
                case '>':
                    $show = $value_a > $value_b;
                    break;
                case '<':
                    $show = $value_a < $value_b;
                    break;
                case '>=':
                    $show = $value_a >= $value_b;
                    break;
                case '<=':
                    $show = $value_a <= $value_b;
                    break;
                default:
                    $show = false;
                    break;
            }

            // Determine show or hide based on condition
            return ($met === 'show') ? $show : !$show;

        }


    }

    add_action('plugins_loaded', array('Itchy_Bricks_Conditional_Display', 'init_plugin'));

endif;